/*
 * idr_var.h 
 * structure and "software" macro definition file for Linux 2.6.x driver for PCI DR11 boards
 *
 * Tahoma Technology
 * (formerly Ikon Corporation)
 * 107 2nd Avenue North
 * Seattle, WA, USA 98109
 *
 * 206.728.6465
 * http://www.tahomatech.com
 * tahoma@tahomatech.com
 * 
 * 
 * 22 February, 1999    start of development - initial target Linux 2.0.35 (R.H. 5.1)
 * 
 * 14 Sept, 1999        re-code for 2.2
 *                      this version no longer compatible with earlier linux revs
 * 
 * 13 March, 2000       started working w/2.3.x
 *                      changed wait_queue type
 * 
 * 24 March, 2000       incorporated kiobuf mechanism for dma to/from user buffers
 *                      replaced copy buf w iopb list to do sg dma
 *                      added pci_dev_int_level to distinguish between the value
 *                      read from the board (to use in restore after eeprom reload)
 *                      and the (possibly) mapped version from pci_dev
 *                      incorporated modified verson of Kaz Kylheku's mutux stuff
 *                      for SMP
 * 
 *                      requires 2.3/2.4 kernel
 *
 * 1 August, 2002	moved conditional variable and mutex struct defines
 *			here from ikon_mutex.h
 *
 * 			deleted * box disclaimer
 *
 * 			added pointer to (possible) copy buffer
 *
 * 			moved "software" macros from idr_reg.h to this file
 *
 *			modified IDR_SG_LINK macro to generate .page correctly
 * 			in HIGHMEM kernel case (not via page_address())
 *
 * 			detect highmem kernel && old scatterlist, and enable
 *			copy buffer pages
 *
 * 			got rid of MODINFBANNER, updated MODULE_DESCRIPTION date
 *
 * 4 February, 2004	major modifications for 2.6 kernel
 *
 * 			new module description date
 *
 * 8 October, 2004	new module description date
 *
 */


#ifndef	_IDR_VAR_H
#define	_IDR_VAR_H
#endif

/*
 * conditional variable and mutex typedefs
 */

typedef struct {
	spinlock_t spin;
	long flags;
} mutex_t;

typedef struct {
	wait_queue_head_t queue;
} cond_t;

enum { COND_WAIT_SUCCESS, COND_WAIT_SIGNAL, COND_WAIT_TIMEOUT };

/*
 * unit structure for boards
 */

struct idr_unit_t {
	u_int instance;			/* equivalent to minor number & board number    */
	mutex_t mutex;			/* used w/cond var to sleep (SMP compatibility) */
	cond_t cond;			/* cond var used w/mutex                        */
	volatile int sleeping;		/* 1 if sleeping (helps detect timeout)         */
	struct pci_dev *pci_dev_p;	/* handle for this board's pci dev structure    */
	uint32_t *plx_page_base_p;	/* virtual address of plx reg page base         */
	uint32_t *idr_page_base_p;	/* virtual address of idr reg page base         */
	volatile uint32_t *plx_base_p;	/* virtual address of plx regs (avoid optimizer)*/
	volatile uint32_t *idr_base_p;	/* virtual address of idr regs (avoid optimizer)*/
	uint32_t *iopb_base_p;		/* virtual add of start of io parameter blocks  */
	int nr_iopbs;			/* number of iopbs				*/
	struct scatterlist *sg_list_p;	/* points to scatterlist (for DMA mappings)     */
	dma_addr_t dma_handle;		/* contains bus address of start of iopbs       */
	struct page **maplist_p;	/* ptr to array of ptrs to page structs         */
	char *maplist_user_buf_p;	/* ptr to current user buffer (or portion of)	*/
	int maplist_nr_pages;		/* number of pages in maplist			*/
	int maplist_offset;		/* offset from base of buffer			*/
	int maplist_length;		/* number of bytes in buffer			*/
	int maplist_rw_mode;		/* READ or WRITE				*/
	char *dma_copy_buf_p;		/* ptr to copy buffer or NULL			*/
	volatile u_int unit_open;	/* 1 = open                                     */
	volatile u_int unit_attached;	/* 1 = attached                                 */
	u_long unit_flags;		/* our flag bits here                           */
	u_long dma_time;		/* dma timeout # in secs                        */
	u_long attn_time;		/* attention wait timeout #                     */
	u_long rdy_time;		/* ready wait timeout #                         */
	u_long read_fcn;		/* use at read start                            */
	u_long write_fcn;		/* use at write start                           */
	u_long read_pulse;		/* pulse at read start                          */
	u_long write_pulse;		/* pulse at write start                         */
	u_long dev_and_vendor_id;	/* device and vendor id from config regs        */
	u_long revision_id;		/* revision id from config regs                 */
	u_int range_resid;		/* remaining count to xfer                      */
					/* may be used for multi-dma                    */
					/* per dr11 blocks (unused?)                    */
	u_int dma_time_def;		/* default dma wait - IN SECONDS                */
	u_int attn_time_def;		/* default attn wait - IN SECONDS               */
	u_int rdy_time_def;		/* default rdy wait - IN SECONDS                */
	u_long read_fcn_def;		/* read fcn default                             */
	u_long write_fcn_def;		/* write fcn default                            */
	u_long read_pulse_def;		/* read pulse default                           */
	u_long write_pulse_def;		/* write pulse default                          */
	u_long endian_def;		/* endian default                               */
	u_long latch_reg_def;		/* latched functions default                    */
	u_long mode_reg_def;		/* mode register default                        */
	u_char int_level;		/* int level read from config register          */
	u_char pci_dev_int_level;	/* (possibly) mapped int level                  */
	char minor_node_name[16];	/* will contain this board's node name          */
};

/*
 * this code was originally placed in the public domain
 * in order to satisfy the "tainted" test, we will declare ourselved GPL'd
 * we certainly meet the spirit of the GPL, since our source is always
 * freely available and included with the driver
 */

MODULE_DESCRIPTION("Tahoma Technology (formerly Ikon Corporation) Pci DR11 Driver - 2004.10.08 11:19");
MODULE_AUTHOR("Tahoma Technology (formerly Ikon Corporation) http://www.tahomatech.com");
MODULE_LICENSE("GPL");

/*
 * debug macro - expands to nothing unless IDR_DEBUG is defined
 * 
 * requires double quoted arg list - printk can take
 * variable # of arguments
 */

#ifdef	IDR_DEBUG
#define	DPRINT(args)	printk args
#else
#define	DPRINT(args)
#endif

/*
 * if user dma mode is permitted by install script (AUTO) and kernel
 * is not configured for more than 4G RAM, use direct user buffer dma.
 * otherwise use a copy buffer
 */

#if	defined(CONFIG_HIGHMEM64G) || defined(IDR_DMA_MODE_COPY)
#define		IDR_COPY_BUF	1
#define		IDR_DMA_MODE_MSG	"user buffer DMA disabled, copy buffer in use\n"
#else
#define		IDR_COPY_BUF	0
#define		IDR_DMA_MODE_MSG	"user buffer DMA enabled\n"
#endif

/*
 * we are 32 bit DMA capable
 */

#define	IDR_DMA_MASK	0xffffffff

/*
 * macro definitions for memory mapped board register access
 * and io control block memory access
 * 
 * these are gathered here to try to allow for easy portability to
 * arch's other than i386, where access to Pci board registers, and
 * the board's access to DMA chain list memory may be byte swizzled
 * 
 * the PLXx macors are used to access plx registers directly
 * the IPLXx macors are used to access the plx dma registers via idr space -
 * which is necessary with early revs of the plx chip - we could use the IDRx macros
 * to do this, but IPLX should be more readable, and it will be easier to find them
 * to mod later if necessary for some future rev of the chip
 * 
 * the configuration accesses will be done explicitly, without macros
 * 
 * unit_p->xxx_base is defined as a pointer to a 32 bit value, offsets are byte offsets, so /4
 * we could also do things with fancy casts
 * 
 * offset to IOPB_XXXX will be iopb # * IOPB_SIZE + byte offset within the iopb so also /4
 *
 * the previous driver version used direct pointer dereferencing (OK on x86)
 * we leave the pointer math as it was rather than try to cast the bases as u_ints or ?
 * to try to avoid future 64 bit pointer issues
 */

#define	PLX_GETL(offset)	readl(unit_p->plx_base_p + ((offset) >> 2))
#define	PLX_PUTL(offset,value)	writel(value, unit_p->plx_base_p + ((offset) >> 2))
#define	IPLX_GETL(offset)	readl(unit_p->idr_base_p + ((offset) >> 2))
#define	IPLX_PUTL(offset,value)	writel(value, unit_p->idr_base_p + ((offset) >> 2))
#define	IDR_GETL(offset)	readl(unit_p->idr_base_p + ((offset) >> 2))
#define	IDR_PUTL(offset,value)	writel(value, unit_p->idr_base_p + ((offset) >> 2))
#define	IOPB_GETL(offset)	readl(unit_p->iopb_base_p + ((offset) >> 2))
#define	IOPB_PUTL(offset,value)	writel(value, unit_p->iopb_base_p + ((offset) >> 2))

/*
 * define size of i/o parameter block (link of dma chain list)
 * and the offsets of elements of the block from the base address
 */

#define IOPB_SIZE               0x10	/* 16 bytes per block   */

#define IOPB_PCI_ADDRESS        0x00	/* starting pci address of user buffer block    */
#define IOPB_LOCAL_ADDRESS      0x04	/* starting local address       */
#define IOPB_TRANSFER_SIZE      0x08	/* number of bytes in this block        */
#define IOPB_NEXT_IOPB          0x0c	/* physical address of next iopb in list        */
